# $Id: LICENCE.pm,v 1.5 2005-04-11 12:24:34 mike Exp $

=head1 NAME

Net::Z3950::DBIServer::LICENCE - the zSQLgate licence

=head1 SYNOPSIS

THIS IS NOT FREE OR OPEN SOURCE SOFTWARE.  You can download, unpack,
build and evaluate it for free, but you may not deploy it without
first purchasing a deployment licence.

=head1 DESCRIPTION

The zSQLgate program and the Net::Z3950::DBIServer module (together
known as "the software") are copyright (C) 2002-2005 Mike Taylor.

I am not a lawyer, so I am not going to try to write this licence in
legalese.  It's supposed to be a clear, English statement of what you
are and are not allowed to do with the software.  Because these terms
are written in plain English, they are no doubt shot through with
loopholes, which I respectfully ask you not to exploit.  I think my
intention is clear, and you should honour that.  If you think my
intention is not clear, then email me for clarification.

1. You are allowed to download this software for evaluation purposes.
   You can unpack it, build it, run it, see how it works and how it
   fits your needs, all at zero cost.  This clause is called your
   EVALUATION LICENCE.

2. You may NOT deploy the software.  For the purposes of this licence,
   deployment means running it for any purpose other than evaluation,
   whether or not you or anyone else makes a profit from doing so.  If
   you wish to deploy the software, you must first contact me and
   arrange to purchase a DEPLOYMENT LICENCE.  If you are unsure
   whether or not your proposed use of the software constitutes
   deployment, email me for clarification.

3. Once you have obtained a deployment licence, you may deploy the
   software in any manner you see fit for the duration of the
   licence, including charging end-users for the services you use it
   to provide.  Multiple deployments require multiple licences.

4. The software remains my property even when a deployment licence has
   been obtained.  A deployment licence does not constitute transfer
   of ownership.

5. You may modify your copy of the software (fix bugs, add features)
   if you need to, whether you are using it under the zero-cost
   evaluation licence or the pay-for deployment licence.  I encourage
   you to send your changes back to me for integration into the
   master copy, but you are not obliged to do so.  You may NOT pass
   your changes on to any other party.

6. There is NO WARRANTY for this software, to the extent permitted by
   applicable law.  I provide the software "as is" without warranty of
   any kind, either expressed or implied, including, but not limited
   to, the implied warranties of MERCHANTABILITY and FITNESS FOR A
   PARTICULAR PURPOSE.  The entire risk as to the quality and
   performance of the software is with you.  Should the software prove
   defective, you assume the cost of all necessary servicing, repair
   or correction.  In no event unless required by applicable law will
   I be liable to you for damages, arising out of the use of the
   software, including but not limited to loss of data or data being
   rendered inaccurate.

7. All rights to the software are reserved by the author except where
   this licence explicitly says otherwise.

=head1 CONTACT DETAILS

The author may be contacted with licence-related queries at
<licence@sql.z3950.org>

=cut
